#!/bin/bash
##################################
# monitoring script
#
# iostat:
#  - gives stats about disk io in system
#
ZBX_REQ_DATA="$2"
ZBX_REQ_DATA_DEV="$1"

#
# Error handling:
#  - need to be displayable in (avoid NOT_SUPPORTED)
#  - items need to be of type "float" (allow negative + float)
#
ERROR_NO_DATA_FILE="-0.9900"
ERROR_OLD_DATA="-0.9901"
ERROR_WRONG_PARAM="-0.9902"
ERROR_MISSING_PARAM="-0.9903"

# Missing device to get data from
if [ -z "$ZBX_REQ_DATA_DEV" ]; then
  echo $ERROR_MISSING_PARAM
  exit 1
fi

# 
# Grab data from SOURCE_DATA for key ZBX_REQ_DATA
#
# 1st check the device exists and gets data gathered by cron job
SOURCE_DATA="iostat -mx"

case $ZBX_REQ_DATA in
  rrqm/s)     $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $2}';;
  wrqm/s)     $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $3}';;
  r/s)        $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $4}';;
  w/s)        $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $5}';;
  rMB/s)      $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $6}';;
  wMB/s)      $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $7}';;
  avgrq-sz)   $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $8}';;
  avgqu-sz)   $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $9}';;
  await)      $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $10}';;
  svctm)      $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $11}';;
  util)       $SOURCE_DATA | grep -E "^$ZBX_REQ_DATA_DEV " | tail -1 | awk '{print $12}';;
  *) echo $ERROR_WRONG_PARAM; exit 1;;
esac
exit 0
